{*******************************************************}
{                                                       }
{         Delphi eXpert Extensions   (X)                }
{                                                       }
{         Copyright (c) 1997 Alexander Buzaev           }
{         FidoNet 2:5020/880.11@fidonet.org             }
{                                                       }
{*******************************************************}
unit ComExp;

interface
Uses
  ExptIntf, VirtIntf, DsgnIntf, ToolIntf, EditIntf, ParsPas,
  StdCtrls, Forms, Classes, Dialogs, SysUtils;

Const
  xVersion='eXpert 1.00';
  xAuthor='Alexander Buzaev';
  xKey='Software\Borland\Delphi\Buzaev';

function GetCurModuleIntf: TIModuleInterface;
function GetActiveForm: TForm;
//Editor
function GetCurEditIntf: TIEditorInterface;
function GetCurrentFile: TStringList;
procedure SetCursorPos(TopPos, CurPos: TEditPos);
function GetBlockAtCursor(szBlock: Integer; var CurPos: Integer): String;
function GetWordAtCursor: String;

implementation

function GetCurModuleIntf: TIModuleInterface;
var
  CurFile: String;
Begin
  Result:=Nil;
  IF ToolServices=Nil Then
    Exit;
  CurFile:=ToolServices.GetCurrentFile;
  IF CompareText(Copy(CurFile, Length(CurFile)-3, 4),'.dfm')=0 Then
    CurFile:=Copy(CurFile, 1, Length(CurFile)-3)+'pas';
  IF CurFile='' Then
    Exit;
  Result:=ToolServices.GetModuleInterface(CurFile);
End;

//    Package
function GetActiveForm: TForm;
var
  CurFile, Value: String;
  FormIntf: TIFormInterface;
  CmpIntf: TIComponentInterface;
  ModuleIntf: TIModuleInterface;
  i: Integer;
begin
  Result:=Nil;
  FormIntf:=Nil; CmpIntf:=Nil;
  ModuleIntf:=GetCurModuleIntf;
  IF ModuleIntf<>Nil Then
  begin
    FormIntf:=ModuleIntf.GetFormInterface;
    IF FormIntf<>Nil Then
    begin
      CmpIntf:=FormIntf.GetFormComponent;
      IF CmpIntf<>Nil Then
      begin
        CmpIntf.GetPropValueByName('Name', Value);
        For i:=0 To Screen.FormCount-1 Do
          IF Screen.Forms[i].Name=Value Then
            Result:=Screen.Forms[i];
        IF Result=Nil Then
          ModuleIntf.ShowForm;
      end;
    end;
  end;
  ModuleIntf.Free;
  FormIntf.Free;
  CmpIntf.Free;
end;

function GetCurEditIntf: TIEditorInterface;
var
  ModuleIntf: TIModuleInterface;
Begin
  ModuleIntf:=Nil; Result:=Nil;
  ModuleIntf:=GetCurModuleIntf;
  IF ModuleIntf<>Nil Then
    Result:=ModuleIntf.GetEditorInterface;
  ModuleIntf.Free;
End;

function GetCurrentFile: TStringList;
const
  FileSize=10240;
var
  s: String;
  Buff: PChar;
  i: Integer;
  EditIntf: TIEditorInterface;
  EdRead: TIEditReader;
Begin
  Result:=Nil;
  EdRead:=Nil;
  EditIntf:=GetCurEditIntf;
  IF EditIntf<>Nil Then
  begin
    EdRead:=EditIntf.CreateReader;
    GetMem(Buff, FileSize);
    s:='';  i:=0;
    While EdRead.GetText(i, Buff, FileSize)=FileSize Do
    begin
      s:=s+Buff;
      Inc(i, FileSize);
    end;
    s:=s+Buff;
    FreeMem(Buff);
    Result:=TStringList.Create;
    Result.Text:=s;
  end;
  EditIntf.Free;
  EdRead.Free;
End;

procedure SetCursorPos(TopPos, CurPos: TEditPos);
var
  CurFile: String;
  EditIntf: TIEditorInterface;
  EditView: TIEditView;
Begin
  IF TopPos.Line<0 Then
    Exit;
  EditView:=Nil;
  EditIntf:=GetCurEditIntf;
  IF EditIntf<>Nil Then
  begin
    EditView:=EditIntf.GetView(0);
    IF EditView<>Nil Then
    begin
      EditView.CursorPos:=CurPos;
      EditView.TopPos:=TopPos;
    end;
  end;
  EditIntf.Free;
  EditView.Free;
End;

function GetWordAtCursor: String;
Const
  szBlock=16;
var
  CurPos, Col, Col0, l: Integer;
  s: String;
Begin
  Result:='';
  s:=GetBlockAtCursor(szBlock, CurPos);
  IF s='' Then
    Exit;
  Col:=CurPos+1;
  l:=Length(s);
  While (Col>0) and not (s[Col] in spWord) do
    Dec(Col);
  Col0:=Col+1;
  Col:=CurPos+1;
  While (Col<=l) and not (s[Col] in spWord) do
    Inc(Col);
  Result:=Copy(s, Col0, Col-Col0);
End;

function GetBlockAtCursor(szBlock: Integer; var CurPos: Integer): String;
var
  Buff: PChar;
  EdIntf: TIEditorInterface;
  EdRead: TIEditReader;
  EdView: TIEditView;
  CharPos: TCharPos;
  EditPos: TEditPos;
  StartPos: Integer;
Begin
  Buff:=Nil;
  EdIntf:=Nil;  EdView:=Nil;  EdRead:=Nil;
  EdIntf:=GetCurEditIntf;
  IF EdIntf<>Nil Then
  begin
    EdView:=EdIntf.GetView(0);
    IF EdView<>Nil Then
    begin
      EditPos:=EdView.CursorPos;
      EdView.ConvertPos(True, EditPos, CharPos);
      EdRead:=EdIntf.CreateReader;
      IF EdRead<>Nil Then
      begin
        GetMem(Buff, szBlock*2);
        StartPos:=EdView.CharPosToPos(CharPos)-szBlock;
        IF StartPos<0 Then
        begin
          StartPos:=0;
          CurPos:=EdView.CharPosToPos(CharPos);
        end
        Else
          CurPos:=szBlock;
        EdRead.GetText(StartPos, Buff, szBlock*2);
        Result:=Copy(Buff,1,szBlock*2);
        FreeMem(Buff, szBlock*2);
      end;//EdRead<>Nil
    end;//IF EdView<>Nil Then
  end;
  EdView.Free;
  EdRead.Free;
  EdIntf.Free;
End;

end.
