{*******************************************************}
{                                                       }
{         Delphi eXtensions Expert(X)                   }
{                                                       }
{         Copyright (c) 1997 Buzaev Alexander           }
{         Field eXtension Expert                        }
{*******************************************************}

unit FldExp;

interface

uses
  Windows, SysUtils, Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, DB, StrUtils,
  DBCtrls, RxDBCtrl, ComExp, Classes, Controls, DBTables;

type
  TFldExpert = class(TForm)
    NB: TNotebook;
    Panel1: TPanel;
    Panel2: TPanel;
    bnNext: TBitBtn;
    bnBack: TBitBtn;
    btCancel: TBitBtn;
    Image1: TImage;
    Image2: TImage;
    Label1: TLabel;
    lbDataSet: TListBox;
    Label2: TLabel;
    Image3: TImage;
    Image4: TImage;
    lbAField: TListBox;
    Image5: TImage;
    Image6: TImage;
    Label6: TLabel;
    cbUseDateEdit: TCheckBox;
    Label8: TLabel;
    Image8: TImage;
    rgLabelPos: TRadioGroup;
    Image7: TImage;
    Label7: TLabel;
    sbAddField: TSpeedButton;
    sbAddAllField: TSpeedButton;
    sbDelField: TSpeedButton;
    sbDelAllField: TSpeedButton;
    Label5: TLabel;
    lbField: TListBox;
    Label4: TLabel;
    Bevel1: TBevel;
    bnCreate: TBitBtn;
    procedure FormActivate(Sender: TObject);
    procedure NBPageChanged(Sender: TObject);
    procedure bnNextClick(Sender: TObject);
    procedure bnBackClick(Sender: TObject);
    procedure sbAddAllFieldClick(Sender: TObject);
    procedure sbAddFieldClick(Sender: TObject);
    procedure sbDelFieldClick(Sender: TObject);
    procedure sbDelAllFieldClick(Sender: TObject);
    procedure bnCreateClick(Sender: TObject);
  private
    ALeft,ATop,AWidth:word;
    AFld:TField;
    ds: TDataSource;
    procedure MoveSelected(List: TCustomListBox; Items: TStrings);
    procedure SetItem(List: TListBox; Index: Integer);
    function  GetFirstSelection(List: TCustomListBox): Integer;
    procedure SetButtons;
  private
    function CreateLabel(s: string): TLabel;
    procedure SetEdParam(ed: TWinControl);
  public
    fm:TForm;
    function ReadDataSet:boolean;
    procedure ReadFields;
  end;

var
  FldExpert: TFldExpert;

procedure ExecuteFldExp;

implementation

{$R *.DFM}

procedure TFldExpert.FormActivate(Sender: TObject);
begin
  bnCreate.Top:=bnNext.Top;
  bnCreate.Left:=bnNext.Left;
  NB.PageIndex:=0;
end;

function TFldExpert.ReadDataSet: Boolean;
var
  i,j: Integer;
  dsName: String;
begin
// DataSet & DataSource  
 For i:=0 To fm.ComponentCount-1 Do
 begin
  IF fm.Components[i] is TDBDataSet Then
  begin
    dsName:='';
    For j:=0 To fm.ComponentCount-1 Do
      IF (fm.Components[j] is TDataSource) and
        (TDataSource(fm.Components[j]).DataSet=fm.Components[i]) Then
      begin
        dsName:=fm.Components[j].Name;
        Break;
      end;
    lbDataSet.Items.Add(Format('%-16s %-16s', [fm.Components[i].Name, dsName]));
  end;
 end;
 Result:=(lbDataSet.Items.Count>0);
 lbDataSet.ItemIndex:=0;
end;

procedure TFldExpert.NBPageChanged(Sender: TObject);
begin
  bnNext.Visible:=True;
  bnCreate.Visible:=False;
  Case NB.PageIndex OF
    1: ReadFields;
    2:
    begin
      bnNext.Visible:=False;
      bnCreate.Visible:=True;
    end;
  End;
end;

procedure TFldExpert.bnNextClick(Sender: TObject);
begin
  NB.PageIndex:=NB.PageIndex+1;
end;

procedure TFldExpert.bnBackClick(Sender: TObject);
begin
  NB.PageIndex:=NB.PageIndex-1;
end;

procedure TFldExpert.ReadFields;
var
  i: Integer;
  ds: TDBDataSet;
begin
  lbField.Items.Clear;
  ds:=TDBDataSet(fm.FindComponent(
    Trim(copy(lbDataSet.Items[lbDataSet.ItemIndex],1,16))));
  ds.Open;  
  For i:=0 To ds.FieldCount-1 Do
    lbField.Items.Add(ds.Fields[i].FieldName);
end;

procedure TFldExpert.sbAddFieldClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := GetFirstSelection(lbAField);
  MoveSelected(lbAField, lbField.Items);
  SetItem(lbAField, Index);
end;

procedure TFldExpert.sbDelFieldClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := GetFirstSelection(lbField);
  MoveSelected(lbField, lbAField.Items);
  SetItem(lbField, Index);
end;

procedure TFldExpert.sbAddAllFieldClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to lbAField.Items.Count - 1 do
    lbField.Items.AddObject(lbAField.Items[I],
      lbAField.Items.Objects[I]);
  lbAField.Items.Clear;
  SetItem(lbAField, 0);
end;

procedure TFldExpert.sbDelAllFieldClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to lbField.Items.Count - 1 do
    lbAField.Items.AddObject(lbField.Items[I], lbField.Items.Objects[I]);
  lbField.Items.Clear;
  SetItem(lbField, 0);
end;

procedure TFldExpert.MoveSelected(List: TCustomListBox; Items: TStrings);
var
  I: Integer;
begin
  for I := List.Items.Count - 1 downto 0 do
    if List.Selected[I] then
    begin
      Items.AddObject(List.Items[I], List.Items.Objects[I]);
      List.Items.Delete(I);
    end;
end;

procedure TFldExpert.SetButtons;
var
  SrcEmpty, DstEmpty: Boolean;
begin
  SrcEmpty := lbAField.Items.Count = 0;
  DstEmpty := lbField.Items.Count = 0;
  sbAddField.Enabled := not SrcEmpty;
  sbAddAllField.Enabled := not SrcEmpty;
  sbDelField.Enabled := not DstEmpty;
  sbDelAllField.Enabled := not DstEmpty;
  bnNext.Enabled:=(lbField.Items.Count>0);
end;

function TFldExpert.GetFirstSelection(List: TCustomListBox): Integer;
begin
  for Result := 0 to List.Items.Count - 1 do
    if List.Selected[Result] then Exit;
  Result := LB_ERR;
end;

procedure TFldExpert.SetItem(List: TListBox; Index: Integer);
var
  MaxIndex: Integer;
begin
  with List do
  begin
    SetFocus;
    MaxIndex := List.Items.Count - 1;
    if Index = LB_ERR then Index := 0
    else if Index > MaxIndex then Index := MaxIndex;
    Selected[Index] := True;
  end;
  SetButtons;
end;

function TFldExpert.CreateLabel(s: string): TLabel;
begin
  Result:=TLabel.Create(fm);
  With Result Do
  try
    Top:=ATop+2;
    Height:=13;
    Caption:=s;
    IF rgLabelPos.ItemIndex=1 Then begin
      AutoSize:=false;
      Width:=ALeft-16;
      Alignment:=taRightJustify;
    end;
    Left:=8;
    Name:='lb'+AFld.FieldName;
    Parent:=fm;
  except
    Free;
    raise;
  end;
end;

procedure TFldExpert.SetEdParam(ed: TWinControl);
begin
 With ed Do
 try
   Top:=ATop;
   Height:=20;
   Width:=AWidth*fm.Font.Size+5;
   Left:=ALeft;
   Parent:=fm;

   Name:='ed'+AFld.FieldName;
 except
   Free;
   raise;
 end;
end;

procedure TFldExpert.bnCreateClick(Sender: TObject);
var
  i: Integer;
  DataSet: TDataSet;
  DBDateEdit: TDBDateEdit;
  DBEdit: TDBEdit;
  lb: TLabel;
begin
  DataSet:=TDataSet(fm.FindComponent(Trim(copy(lbDataSet.Items[lbDataSet.ItemIndex],1, 15))));
  ds:=TDataSource(fm.FindComponent(Trim(copy(lbDataSet.Items[lbDataSet.ItemIndex],15,15))));
  IF ds=Nil Then
  begin
    ds:=TDataSource.Create(fm);
    ds.DataSet:=DataSet;
    i:=1;
    While True Do
    begin
      IF FindComponent('ds'+IntToStr(i))=Nil Then
      begin
        ds.Name:='ds'+IntToStr(i);
        Break;
      end;
      Inc(i);
    end;       
  end;
  ATop:=8;
  ALeft:=0;
  IF rgLabelPos.ItemIndex=1 Then
  begin //  
    for i:=0 to lbField.Items.Count-1 do //   
    IF (length(DataSet.FieldByName(lbField.Items[i]).DisplayLabel) > ALeft) Then
      ALeft:=length(DataSet.FieldByName(lbField.Items[i]).DisplayLabel);
    ALeft:=16+ALeft*fm.Font.Size;
  end
  Else
    ALeft:=8;
    // 
    For i:=0 To lbField.Items.Count-1 Do
    begin
      AFld:=DataSet.FieldByName(lbField.Items[i]);
      lb:=CreateLabel(AFld.DisplayLabel);
      IF rgLabelPos.ItemIndex=0 Then inc(ATop,fm.Font.Size+4+4);
      AWidth:=AFld.DisplayWidth;
      IF (AFld.DataType in [ftDate, ftDateTime]) and (cbUseDateEdit.Checked) Then
      begin
        DBDateEdit:=TDBDateEdit.Create(fm);
        SetEdParam(DBDateEdit);
        DBDateEdit.DataSource:=ds;
        DBDateEdit.DataField:=AFld.FieldName;
        lb.FocusControl:=DBDateEdit;
      end
      Else
      begin
        DBEdit:=TDBEdit.Create(fm);
        SetEdParam(DBEdit);
        DBEdit.DataSource:=ds;
        DBEdit.DataField:=AFld.FieldName;
        lb.FocusControl:=DBEdit;
      end;
      Inc(ATop,fm.Font.Size+12+4);
    end;
  Close;
end;

procedure ExecuteFldExp;
var
  ActiveForm: TForm;
begin
  ActiveForm:=GetActiveForm;
  IF ActiveForm = nil Then
    Exit;
  FldExpert:=TFldExpert.Create(Application);
  try
    FldExpert.fm:=ActiveForm;
    IF FldExpert.ReadDataSet Then
      FldExpert.ShowModal
    Else
      MessageDlg('Must be DBDataSet.', mtError, [mbOK], 0);
  finally
    FldExpert.Free;
  end;
end;


end.
