{*******************************************************}
{                                                       }
{         Delphi eXpert Extensions   (X)                }
{                                                       }
{         Copyright (c) 1997 Alexander Buzaev           }
{         FidoNet 2:5020/880.11@fidonet.org             }
{                                                       }
{*******************************************************}
unit ParsPas;

interface
Uses Classes, SysUtils, EditIntf, Parser, Math;

Const
  spWord=[' ', ',', '.', ';', '=', ':',  '(', ')', #10, #13];
Type
  TMethod = class(TObject)
    MethodName, ObjName: String;
    ProcName, DisplayName: String;
    constructor Create;
  end;

  TParserList = class(TStringList)
  public
    Line,Col: Integer;
    LineImplementation: Integer;
    function EOF: Boolean;
    function GetWord: String;
    //   ,  
    function GetChar: Char;
    procedure Init;
  end;

  TMethodList = class(TList)
  private
    Source: TParserList;
    function  GetMethod(Index: Integer): TMethod;
    procedure SetMethod(Index: Integer; Method: TMethod);
  public
    constructor Create;
    destructor Destroy; override;
    //    
    procedure Parse(const ASource: TStrings);
    function GetMethods(sFilter: String): TStringList;
    function GetMethodPos(Method: TMethod; var TopPos,CurPos: TEditPos): Boolean;
    property Items[Index : Integer]: TMethod read GetMethod write SetMethod; default;
  end;


implementation

//TMethod
constructor TMethod.Create;
Begin
  inherited;
  MethodName:='';  ObjName:='';  ProcName:='';
End;

//////////////////////////////////////////////////////////////////////////////////////////
//TParserList
function TParserList.GetWord: String;
var
  Col0,l: Integer;
Begin
  l:=Length(Strings[Line]);
  While (Col<=l) and (Strings[Line][Col] in spWord) do
    Inc(Col);
  Col0:=Col;
  While (Col<=l) and not (Strings[Line][Col] in spWord) do
    Inc(Col);
  Result:=Copy(Strings[Line], Col0, Col-Col0);
  IF Col>=l Then
  begin
    Col := 1;
    Inc(Line);
  end;
  IF CompareText('Implementation', Result)=0 Then
    LineImplementation:=Line;
End;

function TParserList.GetChar: Char;
var
  l: Integer;
  s: String;
Begin
  l:=Length(Strings[Line]);
  While (Col<=l) and (Strings[Line][Col]=' ') do
    Inc(Col);
  s:=Copy(Strings[Line], Col, 1);
  IF s<>'' Then
    Result:=s[1]
  Else
    Result:=#0;
End;

function TParserList.EOF: Boolean;
Begin
  Result:=(Count<=Line);
End;

procedure TParserList.Init;
Begin
  Line:=0; Col:=1;
End;

//////////////////////////////////////////////////////////////////////////////////////////
//TMethodList
constructor TMethodList.Create;
Begin
  inherited;
  Source:=TParserList.Create;
End;

destructor TMethodList.Destroy;
var
  i: Integer;
Begin
  For i:=0 To Count-1 Do
    Items[i].Free;
  Source.Free;
  inherited;
End;

function  TMethodList.GetMethod(Index: Integer): TMethod;
Begin
  Result:=TMethod(inherited Items[Index]);
End;

procedure TMethodList.SetMethod(Index: Integer; Method: TMethod);
Begin
  inherited Items[Index]:=Method;
End;

function TMethodList.GetMethodPos(Method: TMethod; var TopPos,CurPos: TEditPos): Boolean;
var
  s: String;
  fFind: Boolean;
  ColBegin: Integer;
  procedure Finded;
  begin
    TopPos.Line:=Source.Line-1;
    IF TopPos.Line<1 Then
      TopPos.Line:=1;
    CurPos.Line:=Source.Line+1;
    CurPos.Col:=ColBegin;
    fFind:=True;
  end;
  procedure Find;
  begin
    TopPos.Line:=Source.Line+1;
    Repeat
      s:=Source.GetWord;
      IF CompareText(s, 'begin')=0 Then
      begin
        s:=Source.GetWord;
        CurPos.Line:=Source.Line+1;
        CurPos.Col:=Source.Col-Length(s);
        IF CurPos.Col<1 Then
          CurPos.Col:=1;
        fFind:=True;
        Break;
      end;
    Until (Source.EOF);
  end;//procedure Find;
Begin//procedure TMethodList.GetMethodPos
  TopPos.Col:=1; CurPos.Col:=1;
  TopPos.Line:=-1;
  fFind:=False;
  Source.Init;
  IF not((CompareText('Unit', Method.ObjName)=0) or
    (CompareText('class', Method.ObjName)=0)) Then
      Source.Line:=Source.LineImplementation;
  Repeat
    ColBegin:=Source.Col;
    s:=Source.GetWord;
    IF CompareText(s, Method.ProcName)=0 Then
    begin
      IF CompareText('Unit', Method.ObjName)=0 Then
        Finded
      Else IF CompareText('class', Method.ObjName)=0 Then
      begin
        s:=Source.GetWord;
        IF (CompareText('class', s)=0) and (Source.GetChar='(') Then
        begin
          //  
          Source.Col:=ColBegin;
          Source.GetChar;
          ColBegin:=Source.Col;

          Source.Line:=Source.Line+1;
          Finded;
        end;
      end;
      s:=Source.GetWord;
      IF Method.ObjName<>'' Then
      begin
        IF CompareText(s, Method.ObjName)=0 Then
        begin
          s:=Source.GetWord;
          IF CompareText(s, Method.MethodName)=0 Then
            Find;
        end;
      end
      Else IF CompareText(s, Method.MethodName)=0 Then
      begin//procedure, function...

        Find;
      end;
    end;
  Until Source.EOF or fFind;
  Result:=fFind;
End;

procedure TMethodList.Parse(const ASource: TStrings);
var
  n,i: Integer;
  ch: Char;
  s, s0, s00, ObjName: String;
  fClass: Boolean;
Begin
  Clear;
  IF ASource=Nil Then
    Exit;
  Source.Text:=ASource.Text;
  Source.Init;
  fClass:=False;
  Repeat
    s:=Source.GetWord;
    s00:=s0; s0:=s;
    IF CompareText(s, 'implementation')=0 Then
    begin
      n:=Add(TMethod.Create);
      Items[n].MethodName:=s;
      Items[n].ObjName:='Unit';
      Items[n].ProcName:=s;
      Items[n].DisplayName:=s;
      Break;
    end
    Else IF CompareText(s, 'class')=0 Then
    begin
      i:=Source.Col;
      ch:=Source.GetChar;
      Source.Col:=i;
      IF (ch='(') Then
      begin
        fClass:=True;
        ObjName:=s00;
        n:=Add(TMethod.Create);
        Items[n].MethodName:=ObjName;
        Items[n].ObjName:='class';
        Items[n].ProcName:=ObjName;
        Items[n].DisplayName:=ObjName;
      end
    end
    Else IF (CompareText(s,'end')=0) and fClass Then
    begin
      fClass:=False;
      ObjName:='';
    end
    Else IF (CompareText(s, 'procedure')=0) or (CompareText(s, 'function')=0) or
      (CompareText(s, 'destructor')=0) or (CompareText(s, 'constructor')=0) Then
    begin
      i:=Source.Col;
      ch:=Source.GetChar;
      Source.Col:=i;
      IF (ch<>'(') Then
      begin
        n:=Add(TMethod.Create);
        Items[n].MethodName:=Source.GetWord;
        Items[n].ObjName:=ObjName;
        Items[n].ProcName:=s;
        IF ObjName='' Then
          Items[n].DisplayName:=Items[n].MethodName
        Else
          Items[n].DisplayName:=ObjName+'.'+Items[n].MethodName;
      end;
    end;
  Until (Source.EOF);
End;

function TMethodList.GetMethods(sFilter: String): TStringList;
  function Equal(s1,s2: String): Boolean;
  var
    i: Integer;
  begin
    s1:=UpperCase(s1);  s2:=UpperCase(s2);
    Result:=False;
    IF Length(s2)<Length(s1) Then
      Exit;
    For i:=1 To Length(s1) Do
      IF (s1[i]<>' ') and (s1[i]<>s2[i]) Then
        Exit;
    Result:=True;
  end;
var
  s1, s2: String;
  i: Integer;
Begin//function TMethodList.GetMethods(sFilter: String): TStringList;
  IF (sFilter<>'') and (sFilter[1]='.') Then
    sFilter:=' '+sFilter;
  s1:=GetWord(sFilter, 1, ['.']);
  s2:=GetWord(sFilter, 2, ['.']);
  IF Pos('.', sFilter)=0 Then
  begin
    s2:=s1;
    s1:='';
  end;
  Result:=TStringList.Create;
  For i:=0 To Count-1 Do
    IF Equal(s1, Items[i].ObjName) and Equal(s2, Items[i].MethodName) Then
      Result.AddObject(Items[i].DisplayName, Items[i]);
End;

end.
