{*******************************************************}
{                                                       }
{         Delphi VCL eXtensions   (X)                   }
{                                                       }
{         Copyright (c) 1997 Alexander Buzaev           }
{                                                       }
{*******************************************************}

unit xFastTm;

interface

uses
 Windows, SysUtils, Classes, Controls, Forms;

type
  TXFastTime = class(TComponent)
  private
    c, n1, n2: TLargeInteger;
    Time1, Time2: TDateTime;
    FElapsedTime: String;
    FDisplayControl: TControl;
    FDisplayCaption: String;
    FWaitCursor: Boolean;
    FDecimal: Byte;
  protected
  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
    property ElapsedTime: String read FElapsedTime;
    procedure Start;
    procedure Stop;
    function GetTime: String;
  published
    property Name;
    property Tag;
    property DisplayControl: TControl read FDisplayControl write FDisplayControl;
    property DisplayCaption: String read FDisplayCaption write FDisplayCaption;
    property WaitCursor: Boolean read FWaitCursor write FWaitCursor;
    property Decimal: Byte read FDecimal write FDecimal;
  end;

procedure Register;


implementation
{$R *.DCR}

procedure Register;
begin
  RegisterComponents('X', [TXFastTime]);
end;

Type
  TCControl = class(TControl)
  public
    property Caption;
  end;


constructor TXFastTime.Create(AOwner:TComponent);
begin
  inherited;
  FElapsedTime:='';
  FWaitCursor:=True;
  FDisplayCaption:='Sec: ';
  FDecimal:=2;
end;

destructor TXFastTime.Destroy;
begin
  inherited;
end;

procedure TXFastTime.Start;
begin
  QueryPerformanceFrequency(c);
  IF not QueryPerformanceCounter(n1) Then
    Time1:=Time;
  IF WaitCursor Then
    Screen.Cursor:=crHourGlass;
  IF FDisplayControl<>Nil Then
    With TCControl(FDisplayControl) Do
      Enabled:=False;
end;

function TXFastTime.GetTime: String;
var
  Hour, Min, Sec, MSec: Word;
Begin
  IF not QueryPerformanceCounter(n2) Then
  begin
    Time2:=Time;
    DecodeTime(Time2-Time1, Hour, Min, Sec, MSec);
    Result:=FloatToStrF(Hour*3600+Min*60+Sec+MSec/1000, ffNumber, 16, FDecimal);
  end
  Else
    Result:=FloatToStrF((n2.QuadPart - n1.QuadPart)/
      c.QuadPart, ffNumber, 16, FDecimal);
End;

procedure TXFastTime.Stop;
begin
  FElapsedTime:=DisplayCaption+GetTime;
  IF WaitCursor Then
    Screen.Cursor:=crDefault;
  IF FDisplayControl<>Nil Then
    With TCControl(FDisplayControl) Do
    begin
      Caption:=FElapsedTime;
      Enabled:=True;
    end;
end;

END.

